Package: YAPSA
Type: Package
Title: Yet Another Package for Signature Analysis
Version: 1.36.0
Date: 2025-07-22
Authors@R: c(person("Daniel", "Huebschmann", role = "aut"),
    person("Lea", "Jopp-Saile", role = "aut"),
    person("Carolin", "Andresen", role = "aut"),
    person("Zuguang", "Gu", role = c("aut", "cre"), email = "z.gu@dkfz.de"),
    person("Matthias", "Schlesner", role = "aut"))
Imports: limSolve, SomaticSignatures, VariantAnnotation, Seqinfo,
        reshape2, gridExtra, corrplot, dendextend, GetoptLong,
        circlize, gtrellis, doParallel, parallel, PMCMRplus,
        ggbeeswarm, ComplexHeatmap, KEGGREST, grDevices, Biostrings,
        BSgenome.Hsapiens.UCSC.hg19, magrittr, pracma, dplyr, utils
Depends: R (>= 4.0.0), GenomicRanges, ggplot2, grid
Description: This package provides functions and routines for supervised 
    analyses of mutational signatures (i.e., the signatures have to be known,
    cf. L. Alexandrov et al., Nature 2013 and L. Alexandrov et al., Bioaxiv
    2018). In particular, the family of functions LCD (LCD = linear combination 
    decomposition) can use optimal signature-specific cutoffs which takes care of
    different detectability of the different signatures. Moreover, the package 
    provides different sets of mutational signatures, including the COSMIC and 
    PCAWG SNV signatures and the PCAWG Indel signatures; the latter infering that 
    with YAPSA, the concept of supervised analysis of mutational signatures is 
    extended to Indel signatures. YAPSA also provides confidence intervals as 
    computed by profile likelihoods and can perform signature analysis on a 
    stratified mutational catalogue (SMC = stratify mutational catalogue) in 
    order to analyze enrichment and depletion patterns for the signatures in 
    different strata. 
License: GPL-3
Suggests: testthat, BiocStyle, knitr, rmarkdown
VignetteBuilder: knitr
LazyLoad: yes
biocViews: Sequencing, DNASeq, SomaticMutation, Visualization,
        Clustering, GenomicVariation, StatisticalMethod,
        BiologicalQuestion
RoxygenNote: 7.2.3
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/YAPSA
git_branch: RELEASE_3_22
git_last_commit: 568357f
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 07:16:33 UTC; biocbuild
Author: Daniel Huebschmann [aut],
  Lea Jopp-Saile [aut],
  Carolin Andresen [aut],
  Zuguang Gu [aut, cre],
  Matthias Schlesner [aut]
Maintainer: Zuguang Gu <z.gu@dkfz.de>
