#' Generate R documentation from inline comments.
#'
#' Yet Another Package for mutational Signature analysis
#'
#' This package provides functions and routines useful in 
#' the analysis of mutational signatures (cf. L. Alexandrov et al., Nature 
#' 2013). In particular, functions to perform a signature analysis with 
#' known signatures (\code{\link{LCD}} = linear combination decomposition) and 
#' a signature analysis on stratified mutational catalogue 
#' (\code{\link{run_SMC}} = stratify mutational catalogue) are provided.
#'
#' @docType package
#' @name YAPSA
NULL